/*
 * fxHIIT - A High Intensity Interval Training app for FxOS.
 * Copyright (C) 2015 MiKael NAVARRO
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/* DOMContentLoaded is fired once the document has been loaded and parsed,
   but without waiting for other external resources to load (css/images/etc)
   That makes the app more responsive and perceived as faster.
   https://developer.mozilla.org/Web/Reference/Events/DOMContentLoaded
*/
window.addEventListener('DOMContentLoaded', function() {
    /* We'll ask the browser to use strict code to help us catch errors earlier.
       https://developer.mozilla.org/Web/JavaScript/Reference/Functions_and_function_scope/Strict_mode */
    'use strict';
    
    var translate = navigator.mozL10n.get;
    
    /* We want to wait until the localisations library has loaded all the strings.
       So we'll tell it to let us know once it's ready. */
    navigator.mozL10n.once(start);

    /* Global variables */
    var hiit = [];  // workout plan
    var prep_time = 0;  // prep time/warm up
    var work = 0;  // high intensity work/exercice
    var rest = 0;  // rest/recovery
    var rounds = 0;  // number of rounds
    var cool_down = 0;  // cool down

    var counter = 0;  // timer ID
    var count = 0;  // remaining time (in seconds)
    var total = 0;  // total time (in seconds)
    var cycle = 0;  // current cycle/round

    /* Main/entry point */
    function start() {
	/* We're using textContent because inserting content from external sources into your page 
	   using innerHTML can be dangerous.
	   https://developer.mozilla.org/Web/API/Element.innerHTML#Security_considerations */
	document.getElementById('setup-instructions').textContent = translate('setup_instructions');

	/* Go! */
	document.setup.button.addEventListener('click', function(e) {
	    /* First get workout plan: prep, work, rest, rounds and cool down time */
	    prep_time = calc_seconds(document.setup.preptime.value);
	    work = calc_seconds(document.setup.work.value);
	    rest = calc_seconds(document.setup.rest.value);
	    rounds = parseInt(document.setup.rounds.value);
	    cool_down = calc_seconds(document.setup.cooldown.value);
	    /* Update total time */
	    total = prep_time + (work + rest) * rounds + cool_down;
	    document.setup.total.value = format_countdown(total);

	    /* Compute/expand full HIIT workout plan */
	    hiit = [];
	    hiit.push({'type': "Prep Time", 'duration': prep_time, 'color': "yellow", 'num': ""});
	    for (var n=0; n<rounds; n++) {
		hiit.push({'type': "Work", 'duration': work, 'color': "red", 'num': n+1});
		hiit.push({'type': "Rest", 'duration': rest, 'color': "green", 'num': n+1});
	    }
	    hiit.push({'type': "Cool Down", 'duration': cool_down, 'color': "cyan", 'num': ""});
	    
	    /* Init first workout */
	    cycle = 0;
	    count = hiit[cycle]['duration'];
	    document.getElementById('workout-title').textContent = hiit[cycle]['type'];
	    document.getElementById('countdown').style.color = hiit[cycle]['color'];
	    document.getElementById('countdown').innerHTML = format_countdown(count);
	    document.getElementById('totaltime').innerHTML = format_countdown(total);

	    /* Hide setup-form & show timer */
	    document.setup.style.display = "none";
	    //document.getElementById('workout-title').textContent = translate('workout_title');
	    document.getElementById('workout').style.display = "block";

	    counter = setInterval(timer, 1000);  // 1000 will run it every 1 second
	});

	/* Countdown...
	   http://stackoverflow.com/questions/1191865/code-for-a-simple-javascript-countdown-timer */
	function timer() {
	    count = count - 1;
	    total = total - 1;
	    
	    if (count <= 0) {
		cycle = cycle + 1;  // next cycle
		navigator.vibrate(600);
		document.getElementById('beep').play();

		if (cycle < hiit.length && total > 0) {
		    /* End of cycle: go next */
		    count = hiit[cycle]['duration'];
		    document.getElementById('workout-title').textContent = hiit[cycle]['type'] + " " 
			+ (hiit[cycle]['num']? hiit[cycle]['num'] + "/" + rounds : "");
		    document.getElementById('countdown').style.color = hiit[cycle]['color'];
		}
		else {
		    /* End of workout plan */
		    clearInterval(counter);  // counter ended
		    counter = 0;
		    count = 0;
		    document.getElementById('countdown').innerHTML = "00:00";
		    
		    /* Hide timer & show setup-form */
		    document.getElementById('workout').style.display = "none";
		    document.setup.style.display = "inline-block";
		    
		    return;
		}
	    }
	    /* Show remaining "minutes:seconds" */
	    document.getElementById('countdown').innerHTML = format_countdown(count);
	    document.getElementById('totaltime').innerHTML = format_countdown(total);
	}

	/* Stop */
	document.getElementById('stopbtn').addEventListener('click', function(e) {
	    clearInterval(counter);  // counter reset
	    document.getElementById('countdown').innerHTML = "00:00";
	    
	    /* Hide timer & show setup-form */
	    document.getElementById('workout').style.display = "none";
	    document.setup.style.display = "inline-block";
	});

	/* Pause/resume */
	document.getElementById('pausebtn').addEventListener('click', function(e) {
	    if (counter) {
		clearInterval(counter);  // counter reset
		counter = 0;
		document.getElementById('pausebtn').value = "Resume";
	    }
	    else {
		counter = setInterval(timer, 1000);  // counter restart
		document.getElementById('pausebtn').value = "Pause";
	    }
	});
    }
});

/* Calculate number of seconds (from time "minutes:seconds") */
function calc_seconds(time) {
    var seconds = parseInt(time.split(/(\d+):(\d+)/)[1]) * 60
	    + parseInt(time.split(/(\d+):(\d+)/)[2]);
    return seconds;
}

/* Format time= "minutes:seconds" */
function format_countdown(count) {
    var minutes = ("00" + parseInt(count / 60)).slice(-2);
    var seconds = ("00" + count % 60).slice(-2);
    return minutes + ":" + seconds;
}
